@@ -34,7 +34,7 @@ dnsManagement core and user libs.
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = "0.5"
+__version__ = "0.6"
 
 
 import datetime
@@ -126,10 +126,10 @@ class AuditLog(object):
     try:
       self.db_instance.MakeRow('audit_log', log_dict)
     except:
-      self.db_instance.RollbackTransaction()
+      self.db_instance.EndTransaction(rollback=True)
       raise
 
-    self.db_instance.CommitTransaction()
+    self.db_instance.EndTransaction()
 
 
   def _LogToFile(self, log_string):
