@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.5'
+__version__ = '0.6'
 
 
 import ConfigParser
@@ -69,11 +69,17 @@ class Config(object):
 
     # Supported data types: str, int, boolean, float
     file_schema = {'database': {'server': 'str', 'login': 'str',
-                   'passwd': 'str', 'database': 'str'},
+                                'passwd': 'str', 'database': 'str',
+                                'big_lock_timeout': 'int',
+                                'big_lock_wait': 'int'},
                    'server': {'inf_renew_time': 'int', 'core_die_time': 'int',
-                   'get_credentials_wait_increment': 'int',
-                   'ldap_server': 'str', 'server_killswitch': 'boolean'},
-                   'credentials': {'binddn': 'str', 'exp_time': 'int'}}
+                              'get_credentials_wait_increment': 'int',
+                              'run_as_username': 'str',
+                              'server_killswitch': 'boolean',
+                              'lock_file': 'str', 'ssl_key_file': 'str',
+                              'ssl_cert_file': 'str'},
+                   'credentials': {'authentication_method': 'str',
+                                   'exp_time': 'int'}}
 
     for section in file_schema:
       self.config_file[section] = {}
@@ -99,10 +105,19 @@ class Config(object):
             self.config_file[section][variable] = cp.getboolean(section,
                                                                 variable)
           elif( variables[variable] is 'float' ):
-            self.config_file[section][varaible] = cp.getfloat(section, varaible)
+            self.config_file[section][variable] = cp.getfloat(section, variable)
           else:
             raise ConfigError('DataType "%s" is not supported' % (
-                varaibles[varaible]))
+                variables[variable]))
+
+    if( 'authentication_method' in self.config_file['credentials'] ):
+      authentication_method = self.config_file['credentials'][
+          'authentication_method']
+      self.config_file[authentication_method] = {}
+      authentication_values = cp.items(authentication_method)
+      for authentication_value in authentication_values:
+        self.config_file[authentication_method][authentication_value[0]] = (
+            authentication_value[1])
 
   def GetDb(self):
     """Creates a dbAccess instance.
@@ -113,7 +128,9 @@ class Config(object):
     return db_access.dbAccess(self.config_file['database']['server'],
                               self.config_file['database']['login'],
                               self.config_file['database']['passwd'],
-                              self.config_file['database']['database'])
+                              self.config_file['database']['database'],
+                              self.config_file['database']['big_lock_timeout'],
+                              self.config_file['database']['big_lock_wait'])
 
 
 # vi: set ai aw sw=2:
