@@ -32,13 +32,14 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.5'
+__version__ = '0.6'
 
 
 import datetime
 import IPy
 import constants
 import errors
+import math
 
 class CoreHelpers(object):
   """Library of helper functions that extend the core functions."""
@@ -71,7 +72,18 @@ class CoreHelpers(object):
       string: reverse ip address
     """
     ip_object = IPy.IP(ip_address)
-    return unicode(ip_object.reverseName())
+    reverse_ip_string = ip_object.reverseName()
+    if( ip_object.version() == 4 ):
+      ip_parts = reverse_ip_string.split('.')
+      if( '-' in ip_parts[0] ):
+        range = ip_parts.pop(0).split('-')
+        num_ips = int(range[1]) - int(range[0]) + 1
+        netmask = int(32 - (math.log(num_ips) / math.log(2)))
+        last_octet = ip_parts.pop(0)
+        reverse_ip_string = '.'.join(ip_parts)
+        reverse_ip_string = '%s/%s.%s' % (last_octet, netmask,
+                                          reverse_ip_string)
+    return unicode(reverse_ip_string)
 
   def UnReverseIP(self, ip_address):
     """Un-Reverses reversed IP addresses
@@ -82,10 +94,25 @@ class CoreHelpers(object):
     Outputs:
       string: forward ip address
     """
+    mask = 0
+    cidr_parts = ip_address.split('/')
+    if( len(cidr_parts) == 2 ):
+      mask = cidr_parts[1].split('.')[0]
+      ip_address = ip_address.replace('/%s' % mask, '')
     if( ip_address.endswith('in-addr.arpa.') ):
+      octets = 0
       ip_array = ip_address.split('.')
-      new_ip = '%s.%s.%s.%s' % (ip_array[3], ip_array[2], ip_array[1],
-                                ip_array[0])
+      ip_parts = []
+      while len(ip_array):
+        ip_part = ip_array.pop()
+        if( ip_part.isdigit() ):
+          ip_parts.append(ip_part)
+          octets += 1
+      new_ip = '.'.join(ip_parts)
+      if( mask ):
+        new_ip = '%s/%s' % (new_ip, mask)
+      elif( octets < 4 ):
+        new_ip = '%s/%s' % (new_ip, octets * 8)
     elif( ip_address.endswith('ip6.arpa.') ):
       ip_array = ip_address.split('.')[:-3]
       ip_parts = []
@@ -173,32 +200,53 @@ class CoreHelpers(object):
     self.core_instance.MakeRecord(u'aaaa', target, zone_name, record_args_dict,
                                   view_name, ttl)
 
-  def GetPTRTarget(self, target, view_name=u'any'):
-    """Gets the actual PTR target given view_name.
+  def ExpandIPV6(self, ip_address):
+    """Expands a shorthand ipv6 address to a full ipv6 address
 
     Inputs:
-      target: string of ip address
-      view_name: string of view name
+      ip_address: string of ipv6 address
+
+    Outputs:
+      string: string of long ipv6 address
     """
-    if( not target.endswith('in-addr.arpa.') and not
-        target.endswith('ip6.arpa.') ):
-      target = self.ReverseIP(target)
+    ipv6_address = IPy.IP(ip_address)
+    if( ipv6_address.version() != 6 ):
+      raise errors.CoreError('"%s" is not a valid IPV6 address.' % ipv6_address)
+    return ipv6_address.strFullsize()
+
+  def GetPTRTarget(self, long_target, view_name=u'any'):
+    """Gets the short PTR target given the long PTR target
+
+    Inputs:
+      long_target: String of long PTR target
+      view_name: String of view name
+
+    Ouptuts:
+      string: String of short PTR target
+    """
+    if( not long_target.endswith('in-addr.arpa.') and not
+        long_target.endswith('ip6.arpa.') ):
+      long_target = self.ReverseIP(long_target)
     zone_assignment = None
     reverse_range_zone_assignments = (
         self.core_instance.ListReverseRangeZoneAssignments())
-    ip_address = IPy.IP(self.UnReverseIP(target))
+    ip_address = IPy.IP(self.UnReverseIP(long_target))
     for zone_assignment in reverse_range_zone_assignments:
-      if( ip_address in IPy.IP(
-          reverse_range_zone_assignments[zone_assignment]) ):
-        break
+      if( zone_assignment in reverse_range_zone_assignments ):
+        if( ip_address in IPy.IP(
+            reverse_range_zone_assignments[zone_assignment]) ):
+          break
+    else:
+      raise errors.CoreError(
+          'No suitable reverse range zone assignments found.')
     zone_detail = self.core_instance.ListZones(view_name=view_name)
     zone_origin = zone_detail[zone_assignment][view_name]['zone_origin']
     # Count number of characters in zone origin, add one to count the extra
     # period and remove that number of characters from the target.
     zone_origin_length = len(zone_origin) + 1
-    target = target[:-zone_origin_length:]
+    short_target = long_target[:-zone_origin_length:]
 
-    return (target, zone_assignment)
+    return (short_target, zone_assignment)
 
   def MakePTRRecord(self, target, record_args_dict,
                     view_name=u'any', ttl=None):
@@ -240,24 +288,24 @@ class CoreHelpers(object):
     fwd_record_list = []
     zone_list = []
 
-    ptr_dict = self.core_instance.db_instance.GetEmptyRowDict('records')
+    ptr_dict = self.db_instance.GetEmptyRowDict('records')
     ptr_dict['record_type'] = u'ptr'
-    ptr_args_dict = self.core_instance.db_instance.GetEmptyRowDict(
+    ptr_args_dict = self.db_instance.GetEmptyRowDict(
         'record_arguments_records_assignments')
 
-    zone_dict = self.core_instance.db_instance.GetEmptyRowDict(
+    zone_dict = self.db_instance.GetEmptyRowDict(
         'reverse_range_zone_assignments')
 
-    fwd_dict = self.core_instance.db_instance.GetEmptyRowDict('records')
+    fwd_dict = self.db_instance.GetEmptyRowDict('records')
     fwd_dict['record_type'] = record_type
-    fwd_args_dict = self.core_instance.db_instance.GetEmptyRowDict(
+    fwd_args_dict = self.db_instance.GetEmptyRowDict(
         'record_arguments_records_assignments')
     fwd_args_dict['record_arguments_records_assignments_argument_name'] = (
         u'assignment_ip')
-    self.core_instance.db_instance.StartTransaction()
+    self.db_instance.StartTransaction()
     try:
       reverse_range_zone_assignments_db = (
-          self.core_instance.db_instance.ListRow(
+          self.db_instance.ListRow(
               'reverse_range_zone_assignments', zone_dict))
       for reverse_range_zone_assignment in reverse_range_zone_assignments_db:
         db_zone = reverse_range_zone_assignment[
@@ -271,22 +319,22 @@ class CoreHelpers(object):
           zone_list.append(db_zone)
       for zone in zone_list:
         ptr_dict['record_zone_name'] = zone
-        ptr_record_list.extend(self.core_instance.db_instance.ListRow(
+        ptr_record_list.extend(self.db_instance.ListRow(
             'records', ptr_dict, 'record_arguments_records_assignments',
             ptr_args_dict))
-      num_records = self.core_instance.db_instance.TableRowCount('records')
+      num_records = self.db_instance.TableRowCount('records')
       ratio = num_records / float(user_cidr.len())
       if( ratio > constants.RECORD_RATIO ):
         for ip_address in user_cidr:
           fwd_args_dict['argument_value'] = unicode(ip_address.strFullsize())
-          fwd_record_list.extend(self.core_instance.db_instance.ListRow(
+          fwd_record_list.extend(self.db_instance.ListRow(
               'records', fwd_dict, 'record_arguments_records_assignments',
               fwd_args_dict))
       else:
-        fwd_record_list = self.core_instance.db_instance.ListRow('records',
+        fwd_record_list = self.db_instance.ListRow('records',
             fwd_dict, 'record_arguments_records_assignments', fwd_args_dict)
     finally:
-      self.core_instance.db_instance.CommitTransaction()
+      self.db_instance.EndTransaction()
     records_dict = {}
     for record in ptr_record_list:
       zone_name = record['record_zone_name']
@@ -307,7 +355,7 @@ class CoreHelpers(object):
           records_dict[db_view_name][ip_address] = []
         records_dict[db_view_name][ip_address].append({
             u'forward': False, u'host': record['argument_value'].rstrip('.'),
-            u'zone': record['record_zone_name']})
+            u'zone': record['record_zone_name'], 'zone_origin': zone_origin})
     for record in fwd_record_list:
       ip_address = record['argument_value']
       zone_name = record['record_zone_name']
@@ -323,7 +371,8 @@ class CoreHelpers(object):
         records_dict[view_name][ip_address].append({
             u'forward': True, u'host': '%s.%s' % (
                 record['record_target'], zone_origin.rstrip('.')),
-            u'zone': record['record_zone_name']})
+            u'zone': record['record_zone_name'],
+            u'zone_origin': zone_origin})
     return records_dict
 
   def ListNamedConfGlobalOptionsClient(self, option_id=None,
@@ -377,22 +426,22 @@ class CoreHelpers(object):
       int: number of rows modified
     """
     record_arguments_record_assignments_dict = (
-        self.core_instance.db_instance.GetEmptyRowDict(
+        self.db_instance.GetEmptyRowDict(
             'record_arguments_records_assignments'))
     record_arguments_record_assignments_dict[
         'record_arguments_records_assignments_type'] = u'cname'
     record_arguments_record_assignments_dict[
         'argument_value'] = hostname
-    records_dict = self.core_instance.db_instance.GetEmptyRowDict(
+    records_dict = self.db_instance.GetEmptyRowDict(
         'records')
     records_dict['record_type'] = u'cname'
     records_dict['record_view_dependency'] = '%s_dep' % view_name
     records_dict['record_zone_name'] = zone_name
     success = False
     try:
-      self.core_instance.db_instance.StartTransaction()
+      self.db_instance.StartTransaction()
       try:
-        found_record_arguments = self.core_instance.db_instance.ListRow(
+        found_record_arguments = self.db_instance.ListRow(
             'record_arguments_records_assignments',
             record_arguments_record_assignments_dict)
         remove_record_dict = {}
@@ -403,7 +452,7 @@ class CoreHelpers(object):
         row_count = 0
         for record_id in remove_record_dict:
           records_dict['records_id'] = record_id
-          found_records_dict = self.core_instance.db_instance.ListRow(
+          found_records_dict = self.db_instance.ListRow(
               'records', records_dict)
           if( len(found_records_dict) != 1 ):
             raise errors.CoreError('Incorrect number of records found!')
@@ -412,14 +461,14 @@ class CoreHelpers(object):
                 'RemoveRecord', target=found_records_dict[0]['record_target'])
           except self.core_instance.user_instance.AuthError:
             continue
-          row_count += self.core_instance.db_instance.RemoveRow(
+          row_count += self.db_instance.RemoveRow(
               'records', found_records_dict[0])
           remove_record_dict[record_id].update({
               'cname_host': found_records_dict[0]['record_target']})
       except:
-        self.core_instance.db_instance.RollbackTransaction()
+        self.db_instance.EndTransaction(rollback=True)
         raise
-      self.core_instance.db_instance.CommitTransaction()
+      self.db_instance.EndTransaction()
       success = True
       remove_record_string = ''
       log_list = []
@@ -563,9 +612,9 @@ class CoreHelpers(object):
                                              record['record_zone_name'])
 
       except:
-        self.db_instance.RollbackTransaction()
+        self.db_instance.EndTransaction(rollback=True)
         raise
-      self.db_instance.CommitTransaction()
+      self.db_instance.EndTransaction()
       success = True
     finally:
       for operation in log_dict:
@@ -581,3 +630,7 @@ class CoreHelpers(object):
                                            record['record_arguments'],
                                            record['view_name']), success)
     return row_count
+
+  def CIDRtoOrigin(self, cidr_block):
+    ip = IPy.IP(cidr_block)
+
