@@ -56,9 +56,9 @@ The two primary uses of this class are:
     for row in matching_rows:
       db_instance.RemoveRow('acls', row)
   except Exception:
-    db_instance.RollbackTransaction()
+    db_instance.EndTransaction(rollback=True)
   else:
-    db_instance.CommitTransaction()
+    db_instance.EndTransaction()
 
 Note: MySQLdb.Error can be raised in almost any function in this module. Please
       keep that in mind when using this module.
@@ -66,9 +66,15 @@ Note: MySQLdb.Error can be raised in almost any function in this module. Please
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.5'
+__version__ = '0.6'
 
 
+import datetime
+import Queue
+import threading
+import time
+import uuid
+
 import MySQLdb
 import MySQLdb.cursors
 
@@ -95,13 +101,31 @@ class MissingDataTypeError(data_validation.MissingDataTypeError):
 
 
 class dbAccess(object):
+  """This class provides the primary interface for connecting and interacting
+   with the roster database.
+   """
 
-  def __init__(self, db_host, db_user, db_passwd, db_name):
+  def __init__(self, db_host, db_user, db_passwd, db_name, big_lock_timeout,
+               big_lock_wait, thread_safe=True):
+    """Instantiates the db_access class.
+
+    Inputs:
+      db_host: string of the database host name
+      db_user: string of the user name used to connect to mysql
+      db_passwd: string of password used to connect to mysql
+      db_name: string of name of database in mysql server to use
+      big_lock_timeout: integer of how long the big lock should be valid for
+      big_lock_wait: integer of how long to wait for proccesses to finish
+                     before locking the database
+      thread_safe: boolean of if db_acceess should be thread safe
+    """
     # Do some better checking of these args
     self.db_host = db_host
     self.db_user = db_user
     self.db_passwd = db_passwd
     self.db_name = db_name
+    self.big_lock_timeout = big_lock_timeout
+    self.big_lock_wait = big_lock_wait
     self.transaction_init = False
     self.connection = None
     self.cursor = None
@@ -110,26 +134,50 @@ class dbAccess(object):
     self.foreign_keys = []
     self.data_validation_instance = None
     self.locked_db = False
-    
+    self.thread_safe = thread_safe
+    self.queue = Queue.Queue()
+    self.now_serving = None
+    self.queue_update_lock = threading.Lock()
 
-  def close(self): # Name this function better
+  def close(self):
+    """Closes a connection that has been opened. 
+
+    A new connection will be created on StartTransaction.
+    """
     if( self.connection is not None ):
       self.connection.close()
-      self.connection = None
-
+    self.connection = None
+    
   def StartTransaction(self):
     """Starts a transaction.
 
     Also it starts a db connection if none exists or it times out.
     Always creates a new cursor.
+    
+    This function also serializes all requests on this object and if the 
+    big lock has been activated will wait for it to be released.
 
     Raises:
       TransactionError: Cannot start new transaction last transaction not
                         committed or rolled-back.
     """
-    if( self.transaction_init ):
-      raise TransactionError('Cannot start new transaction last transaction'
-                             'not committed or rolled-back.')
+    if( self. thread_safe ):
+      unique_id = uuid.uuid4()
+      self.queue.put(unique_id)
+
+      while_sleep = 0
+      while( unique_id != self.now_serving ):
+        time.sleep(while_sleep)
+        self.queue_update_lock.acquire()
+        if( self.now_serving is None ):
+          self.now_serving = self.queue.get()
+        self.queue_update_lock.release()
+        while_sleep = 0.005
+
+    else:
+      if( self.transaction_init ):
+        raise TransactionError('Cannot start new transaction last transaction '
+                               'not committed or rolled-back.')
 
     if( self.connection is not None ):
       try:
@@ -143,35 +191,59 @@ class dbAccess(object):
                                         use_unicode=True)
       self.cursor = self.connection.cursor(MySQLdb.cursors.DictCursor)
 
+    while_sleep = 0
+    db_lock_locked = 1
+    while( db_lock_locked ):
+      time.sleep(while_sleep)
+      try:
+        self.cursor.execute('SELECT `locked`, `lock_last_updated`, '
+                            'NOW() as `now` from `locks` WHERE '
+                            '`lock_name`="db_lock_lock"')
+        rows = self.cursor.fetchall()
+      except MySQLdb.ProgrammingError:
+        break
+      if( not rows ):
+        break
+      lock_last_updated = rows[0]['lock_last_updated']
+      db_lock_locked = rows[0]['locked']
+      now = rows[0]['now']
+      if( (now - lock_last_updated).seconds > self.big_lock_timeout ):
+        break
+      while_sleep = 1
+
     self.transaction_init = True
 
-  def CommitTransaction(self):
-    """Commits a transaction.
+  def EndTransaction(self, rollback=False):
+    """Ends a transaction.
 
-    Also does some simple checking to make sure a connection was open first.
+    Also does some simple checking to make sure a connection was open first
+    and releases itself from the current queue.
+
+    Inputs:
+      rollback: boolean of if the transaction should be rolled back
 
     Raises:
-      TransactionError: Must run StartTansaction before roll-back.
+      TransactionError: Must run StartTansaction before EndTransaction.
     """
-    if( not self.transaction_init ):
-      raise TransactionError('Must run StartTansaction before commit.')
-    self.cursor.close()
-    self.connection.commit()
-    self.transaction_init = False
+    if( not self.thread_safe ):
+      if( not self.transaction_init ):
+        raise TransactionError('Must run StartTansaction before '
+                               'EndTransaction.')
 
-  def RollbackTransaction(self):
-    """Rollback a transaction.
+    try:
+      self.cursor.close()
+      if( rollback ):
+        self.connection.rollback()
+      else:
+        self.connection.commit()
 
-    Also does some simple checking to make sure a connection was open first.
-
-    Raises:
-      TransactionError: Must run StartTansaction before roll-back.
-    """
-    if( not self.transaction_init ):
-      raise TransactionError('Must run StartTansaction before roll-back.')
-    self.cursor.close()
-    self.connection.rollback()
-    self.transaction_init = False
+    finally:
+      self.transaction_init = False
+      if( self.thread_safe ):
+        if( not self.queue.empty() ):
+          self.now_serving = self.queue.get()
+        else:
+          self.now_serving = None
 
   def LockDb(self):
     """This function is to lock the whole database for consistent data
@@ -182,6 +254,9 @@ class dbAccess(object):
     """
     if( self.locked_db is True ):
       raise TransactionError('Must unlock tables before re-locking them')
+    self.cursor.execute('UPDATE `locks` SET `locked`=1 WHERE '
+                        '`lock_name`="db_lock_lock"')
+    time.sleep(self.big_lock_wait)
     self.cursor.execute(
         'LOCK TABLES %s READ' % ' READ, '.join(constants.TABLES.keys()))
     self.locked_db = True
@@ -195,6 +270,8 @@ class dbAccess(object):
     if( self.locked_db is False ):
       raise TransactionError('Must lock tables before unlocking them')
     self.cursor.execute('UNLOCK TABLES')
+    self.cursor.execute('UPDATE `locks` SET `locked`=0 WHERE '
+                        '`lock_name`="db_lock_lock"')
     self.locked_db = False
 
   def InitDataValidation(self):
@@ -285,7 +362,7 @@ class dbAccess(object):
         else:
           return {}
     finally:
-      self.CommitTransaction()
+      self.EndTransaction()
 
     auth_info_dict['user_access_level'] = db_data[0]['access_level']
     for row in db_data:
@@ -612,7 +689,7 @@ class dbAccess(object):
       record_arguments = self.ListRow('record_arguments',
                                       search_record_arguments_dict)
     finally:
-      self.CommitTransaction()
+      self.EndTransaction()
 
     record_arguments_dict = {}
     if( not record_arguments ):
